/*
 * Copyright (c) 2007 Michal Kebrt
 * Copyright (c) 2007 Petr Stepan
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup arm32	
 * @{
 */
/** @file
 */

#ifndef KERN_arm32_EXCEPTION_H_
#define KERN_arm32_EXCEPTION_H_

#include <arch/types.h>

/* Exception Vectors */
#define EXC_RESET_VEC		0x0
#define EXC_UNDEF_INSTR_VEC	0x4
#define EXC_SWI_VEC		0x8
#define EXC_PREFETCH_ABORT_VEC	0xc
#define EXC_DATA_ABORT_VEC	0x10
#define EXC_IRQ_VEC		0x18
#define EXC_FIQ_VEC		0x1c

typedef struct {
	
	uint32_t cpsr;
	uint32_t retaddr;

	uint32_t r0;
	uint32_t r1;
	uint32_t r2;
	uint32_t r3;
	uint32_t r4;
	uint32_t r5;
	uint32_t r6;
	uint32_t r7;
	uint32_t r8;
	uint32_t r10;
	uint32_t r11;
	uint32_t r12;
	uint32_t sp;
	uint32_t lr;

} istate_t;

static inline void istate_set_retaddr(istate_t *istate, uintptr_t retaddr)
{
	istate->retaddr = retaddr;
}

/** Return true if exception happened while in userspace */
static inline int istate_from_uspace(istate_t *istate)
{
	return !(istate->retaddr & 0x80000000);
}
static inline unative_t istate_get_pc(istate_t *istate)
{
	return istate->retaddr;
}


extern void install_exception_handlers(void);
extern void exception_init(void);

#endif

/** @}
 */
